#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/pgmspace.h>
#include <stdio.h>
#include <stdlib.h>
#include "lcd.h"

#define SUS 1
#define STG 2
#define DRT 3
#define JOS 4

#define CAP 1
#define COADA 2
#define HRANA 3

#define SIZE 2
#define INIT_LEN 4

typedef struct{
	char x;
	char y;
	char type;
} Block;


Block snake[100];
Block food;
char directia;
int lungime;
int scor = 0;
int vieti = 3;


void drawBlocks(void){
	int i, j, k;
	char x1, x2, y1, y2;
	for (i = 0; i < lungime; i++){
		x1 = snake[i].x;
		y1 = snake[i].y;
		x2 = snake[i].x + SIZE;
		y2 = snake[i].y + SIZE;
		for(j = x1; j < x2; j++)
			for(k = y1; k < y2; k++)
				lcd_setpixel(j, k);
	}
	x1 = food.x;
	y1 = food.y;
	x2 = food.x + SIZE;
	y2 = food.y + SIZE;
	for(j = x1; j < x2; j++)
		for(k = y1; k < y2; k++)
			lcd_setpixel(j, k);
}
void snake_init(void){
	int i;
	lungime = INIT_LEN;
	directia = DRT;
	snake[0].type = CAP;
	snake[0].x = 10;
	snake[0].y = 10;
	for (i = 1; i < lungime; i++){
		snake[i].type = COADA;
		snake[i].x = snake[0].x - i * SIZE;
		snake[i].y = snake[0].y;
	}
	food.x = rand()%(LCD_X_RES - SIZE);
	food.y = rand()%(LCD_Y_RES - SIZE);
	food.type = HRANA;
}

void snake_depl(char dir){
	int i;
	for (i = lungime - 1; i > 0; i--){
		snake[i].x = snake[i - 1].x; 
		snake[i].y = snake[i - 1].y;
	}
	if (dir == SUS)
		if (snake[0].y > 0)
			snake[0].y -= SIZE;
		else
			snake[0].y = LCD_Y_RES - SIZE;
	else if (dir == JOS)
		if (snake[0].y < LCD_Y_RES - SIZE)
			snake[0].y += SIZE;
		else
			snake[0].y = 0;
	else if (dir == STG)
		if (snake[0].x > 0)
			snake[0].x -= SIZE;
		else
			snake[0].x = LCD_X_RES - SIZE;
	else if (dir == DRT)
		if (snake[0].x < LCD_X_RES - SIZE)
			snake[0].x += SIZE;
		else
			snake[0].x = 0;
}

void check_snake(void){
	int i;
	for(i = 1; i < lungime; i++){
		if ((snake[0].x == snake[i].x) && (snake[0].y == snake[i].y)){
			vieti--;
			if (vieti > 0)
				snake_init();
			else 
				break;
		}
	}
	if (((snake[0].x <= food.x + SIZE) && (snake[0].y <= food.y + SIZE) && (snake[0].x >= food.x) && (snake[0].y >= food.y)) ||
		((snake[0].x + SIZE <= food.x + SIZE) && (snake[0].y + SIZE <= food.y + SIZE) && (snake[0].x + SIZE >= food.x) && (snake[0].y + SIZE >= food.y))){
		snake[lungime].x = snake[lungime - 1].x;
		snake[lungime].y = snake[lungime - 1].y;
		snake[lungime].type = COADA;
		lungime++;
		scor++;
		food.x = rand()%(LCD_X_RES - SIZE);
		food.y = rand()%(LCD_Y_RES - SIZE);
	}
}

void game_over(){
	char buf[2];
	itoa (scor, buf, 10);
	lcd_init();
	lcd_clear();
	lcd_update();
	lcd_gotoXY(11, 1);	
	lcd_str("Game Over", 9);
	lcd_gotoXY(15, 2);	
	lcd_str("SCOR:", 5);
	lcd_gotoXY(50, 2);
	if (scor < 9)
		lcd_str(buf, 1);
	else 
		lcd_str(buf, 2);
	_delay_ms(15000);
	lcd_clear();
	lcd_update();
}

/* main -- Program starts here */
void SnakeDelay(void){
	volatile int i;
	for ( i = -32000; i < 32000; i++ );
}
int main(void)
{	unsigned char newDirection;
	srand(25);
	lcd_init();
	lcd_clear();
	lcd_update();
	lcd_gotoXY(25, 1);	
	lcd_str("SNAKE", 5);
	lcd_gotoXY(36, 2);	
	lcd_str("by", 2);
	lcd_gotoXY(11, 3);	
	lcd_str("Valentina", 9);
	_delay_ms(15000);	
	
	DDRC = 0;
	PORTC = 0xFF;

	snake_init();

	while(vieti > 0){
		lcd_clear();
		newDirection = directia;
		if (!(PINC&(1<<PC2))) newDirection = SUS;
		if (!(PINC&(1<<PC3))) newDirection = JOS;
		if (!(PINC&(1<<PC4))) newDirection = STG;
		if (!(PINC&(1<<PC5))) newDirection = DRT;		
		if ((newDirection == SUS) && (directia != JOS)) directia = newDirection;
		if ((newDirection == JOS) && (directia != SUS)) directia = newDirection;
		if ((newDirection == STG) && (directia != DRT)) directia = newDirection;
		if ((newDirection == DRT) && (directia != STG)) directia = newDirection;				
		snake_depl(directia);
		check_snake();
		drawBlocks();
		SnakeDelay();
		lcd_update();
	}
	game_over();
}